@@ -32,10 +32,10 @@ bool Coff::getName(const QByteArray &data, uint &offset, uint nbChars, uint stri
 }
 
 const Coff::OptHeaderFormat::Data Coff::OptHeaderFormat::DATA[Nb_Types] = {
-  { 0, I18N_NOOP("Old Microchip") },
-  { 0, I18N_NOOP("New Microchip") },
-  { 0, I18N_NOOP("PICC Compiler") },
-  { 0, I18N_NOOP("CCS Compiler")  }
+  { I18N_NOOP("Old Microchip") },
+  { I18N_NOOP("New Microchip") },
+  { I18N_NOOP("PICC Compiler") },
+  { I18N_NOOP("CCS Compiler")  }
 };
 
 const Coff::OptHeaderData Coff::OPT_HEADER_DATA[] = {
@@ -49,10 +49,10 @@ const Coff::OptHeaderData Coff::OPT_HEADER_DATA[] = {
 
 //----------------------------------------------------------------------------
 const Coff::AuxSymbolType::Data Coff::AuxSymbolType::DATA[Nb_Types] = {
-  { 0, I18N_NOOP("Direct") },
-  { 0, I18N_NOOP("File") },
-  { 0, I18N_NOOP("Indentifier") },
-  { 0, I18N_NOOP("Section") }
+  { I18N_NOOP("Direct") },
+  { I18N_NOOP("File") },
+  { I18N_NOOP("Indentifier") },
+  { I18N_NOOP("Section") }
 };
 
 Coff::AuxSymbol *Coff::AuxSymbol::factory(const Object &object, AuxSymbolType type, const QByteArray &data, uint offset, uint stringTableOffset, Log::Base &log)
@@ -119,10 +119,10 @@ Coff::AuxSymbolSection::AuxSymbolSection(const Object &object, const QByteArray
 
 //----------------------------------------------------------------------------
 const Coff::SymbolSectionType::Data Coff::SymbolSectionType::DATA[Nb_Types] = {
-  { 0, I18N_NOOP("Inside Section") },
-  { 0, I18N_NOOP("Undefined Section") },
-  { 0, I18N_NOOP("Absolute Value") },
-  { 0, I18N_NOOP("Debug Symbol") }
+  { I18N_NOOP("Inside Section") },
+  { I18N_NOOP("Undefined Section") },
+  { I18N_NOOP("Absolute Value") },
+  { I18N_NOOP("Debug Symbol") }
 };
 
 const Coff::SymbolClass::Data Coff::SymbolClass::DATA[Nb_Types] = {
@@ -205,7 +205,7 @@ Coff::Symbol::Symbol(const Object &object, const QByteArray &data, uint start,
     if ( v==sclass.data().id ) { _sclass = sclass; break; }
   if ( !getULong(data, offset, 1, log, v) ) return;
   uint nbAux = v;
-  //qDebug("symbol: %s value=%s type=%i dtype=%i class=%i nbAux=%i section=%i", _name.latin1(), toHexLabel(_value, 4).latin1(), _type, _dtype, _class, nbAux, _section);
+  log.log(Log::DebugLevel::Max, QString("symbol \"%1\": value=%2 type=%3 dtype=%4 class=%5 nbAux=%6 section=%7").arg(_name).arg(toHexLabel(_value, 2*4)).arg(_type.type()).arg(_dtype.type()).arg(_sclass.type()).arg(nbAux).arg(_section));
 
   AuxSymbolType auxType = AuxSymbolType::Nb_Types;
   if ( _name==".direct" ) auxType = AuxSymbolType::Direct;
@@ -323,13 +323,13 @@ Coff::CodeLine::CodeLine(const Object &object, const Section &section,
 
 //----------------------------------------------------------------------------
 const Coff::SectionType::Data Coff::SectionType::DATA[Nb_Types] = {
-  { 0, I18N_NOOP("Config") },
-  { 0, I18N_NOOP("Device ID") },
-  { 0, I18N_NOOP("User IDs") },
-  { 0, I18N_NOOP("Uninitialized Data") },
-  { 0, I18N_NOOP("Initialized Data") },
-  { 0, I18N_NOOP("Rom Data") },
-  { 0, I18N_NOOP("Code") }
+  { I18N_NOOP("Config") },
+  { I18N_NOOP("Device ID") },
+  { I18N_NOOP("User IDs") },
+  { I18N_NOOP("Uninitialized Data") },
+  { I18N_NOOP("Initialized Data") },
+  { I18N_NOOP("Rom Data") },
+  { I18N_NOOP("Code") }
 };
 
 Coff::Section::Section(const Device::Data &device, const Object &object,
@@ -362,7 +362,7 @@ Coff::Section::Section(const Device::Data &device, const Object &object,
   // read data
   Q_ASSERT ( device.group().name()=="pic" );
   const Pic::Data &pdata = static_cast<const Pic::Data &>(device);
-  //qDebug("section %s: address=%s size=%i flags=%i", _name.data(), toHexLabel(_address, 4).latin1(), _size, int(_flags));
+  log.log(Log::DebugLevel::Max, QString("section \"%1\": address=%2 size=%3 flags=%4").arg(_name).arg(toHexLabel(_address, 2*4)).arg(_size).arg(_flags));
   if ( _size!=0 && dataOffset!=0 ) {
     uint inc = 1;
     uint nbWords = _size;
@@ -512,11 +512,20 @@ bool Coff::Object::parse(Log::Base &log)
   for (uint i=0; i<nbSymbols(); i++) {
     if ( _symbols[i]==0 || _symbols[i]->isAuxSymbol() ) continue;
     const Symbol *sym = static_cast<const Symbol *>(_symbols[i]);
-    if ( sym->symbolClass()!=SymbolClass::Static ) continue;
+    if ( sym->symbolClass()!=SymbolClass::Static && sym->symbolClass()!=SymbolClass::External ) continue;
     if ( sym->sectionType()!=SymbolSectionType::InsideSection ) continue;
-    QString name = sym->name();
-    if ( name.startsWith("_$_") || name.startsWith("__") || name.startsWith(".") ) continue; // special variables (?)
-    _variables[name] = sym->value() & 0xFFF; // #### ??
+    SectionType type = _sections[sym->section()]->type();
+    if ( type!=SectionType::UninitializedData && type!=SectionType::InitializedData ) continue;
+    if ( _device!=0 && _device->group().name()=="pic" ) {
+      const Pic::RegistersData &rdata = static_cast<const Pic::Data *>(_device)->registersData();
+      if ( rdata.type(sym->value())==Pic::Sfr ) continue;
+    }
+    uint flags = _sections[sym->section()]->flags();
+    if ( flags & Section::FOverlay ) continue; // ### get rid of some registers (SDCC) ??
+    if ( sym->name().startsWith("__") ) continue; // remove internal variables (C18)
+    QString name = _sections[sym->section()]->name();
+    if ( name.contains("shadowregs") || name.startsWith("UDL_idata") ) continue; // remove more (SDCC)
+    _variables[sym->name()] = sym->value() & 0xFFF; // ### why the mask ??
   }
 
   return true;
@@ -603,7 +612,7 @@ QValueVector<Pic::RegisterNameData> Pic::sfrList(const Pic::Data &data)
   QValueVector<Pic::RegisterNameData> list;
   const Pic::RegistersData &rdata = data.registersData();
   for (uint i=0; i<rdata.nbRegisters(); i++) {
-    uint address = rdata.nbBytes() * i;
+    uint address = rdata.addressFromIndex(i);
     Pic::RegisterType type = rdata.type(address);
     Device::RegisterProperties rp = rdata.properties(address);
     if ( !(rp & Device::Readable) ) continue;
@@ -621,32 +630,24 @@ QValueVector<Pic::RegisterNameData> Pic::sfrList(const Pic::Data &data)
   return list;
 }
 
-QValueVector<Pic::RegisterNameData> Pic::gprList(const Pic::Data &data, const Coff::Object *coff)
+Pic::RegisterNameData Pic::gprNameData(const Pic::RegistersData &rdata, uint index, const Coff::Object *coff)
 {
-  QValueVector<Pic::RegisterNameData> list;
-  const Pic::RegistersData &rdata = data.registersData();
-  for (uint i=0; i<rdata.nbRegisters(); i++) {
-    uint address = rdata.nbBytes() * i;
-    Pic::RegisterType type = rdata.type(address);
-    Device::RegisterProperties rp = rdata.properties(address);
-    if ( !(rp & Device::Readable) ) continue;
-    if (type==Pic::Gpr ) {
-      QString s = toHexLabel(address, rdata.nbCharsAddress());
-      if (coff) {
-        QString name = coff->variableName(address);
-        if ( !name.isEmpty() ) s += " (" + name + ")";
-      }
-      Register::TypeData rtd(address, rdata.nbChars());
-      list.append(Pic::RegisterNameData(s, rtd));
-    }
+  uint address = rdata.addressFromIndex(index);
+  Pic::RegisterType type = rdata.type(address);
+  Device::RegisterProperties rp = rdata.properties(address);
+  if ( !(rp & Device::Readable) || type!=Pic::Gpr ) return Pic::RegisterNameData();
+  QString s = toHexLabel(address, rdata.nbCharsAddress());
+  if (coff) {
+    QString name = coff->variableName(address);
+    if ( !name.isEmpty() ) s += " (" + name + ")";
   }
-  return list;
+  Register::TypeData rtd(address, rdata.nbChars());
+  return Pic::RegisterNameData(s, rtd);
 }
 
-QValueVector<Pic::RegisterNameData> Pic::variableList(const Pic::Data &data, const Coff::Object &coff)
+QValueVector<Pic::RegisterNameData> Pic::variableList(const Pic::RegistersData &rdata, const Coff::Object &coff)
 {
   QValueVector<Pic::RegisterNameData> list;
-  const Pic::RegistersData &rdata = data.registersData();
   QMap<QString, Address> variables = coff.variables();
   QMap<QString, Address>::const_iterator vit;
   for (vit=variables.begin(); vit!=variables.end(); ++vit) {
