@@ -11,9 +11,9 @@
 #include <qregexp.h>
 
 const Pic::ConfigNameType::Data Pic::ConfigNameType::DATA[Nb_Types] = {
-  { "cname", 0 },
-  { "ecname", 0 },
-  { "sdcc_cname", 0 }
+  { "cname"      },
+  { "ecname"     },
+  { "sdcc_cname" }
 };
 
 const Pic::Config::Data Pic::Config::DATA[] = {
@@ -191,7 +191,7 @@ const Pic::Config::Data Pic::Config::DATA[] = {
     { "0", I18N_NOOP("Disabled") }, { 0, 0 } } },
   { { "GCP", I18N_NOOP("General code segment read-protection") }, MemoryRange, { { 0, 0 } } },
   { { "GWRP", I18N_NOOP("General code segment write-protection") }, MemoryRange, { { 0, 0 } } },
-  { { "COE", I18N_NOOP("Reset into clip on emulation mode") }, Toggle, { { 0, 0 } } },
+  { { "COE", I18N_NOOP("Reset into clip-on emulation mode") }, Toggle, { { 0, 0 } } },
   { { "ICS", I18N_NOOP("ICD communication channel") }, Pins, { { 0, 0 } } },
 
   { { "USBDIV", I18N_NOOP("USB clock (PLL divided by)") }, ValueUInt, {
