@@ -85,7 +85,7 @@ BitValue Pic::Memory::normalizedWord(MemoryRangeType type, uint offset) const
 
 void Pic::Memory::setWord(MemoryRangeType type, uint offset, BitValue value)
 {
-  if ( offset>=_ranges[type].size() ) qDebug("Memory::setWord: type=%s offset=%s size=%s value=%s", type.key(), toHexLabelAbs(offset).latin1(), toHexLabelAbs(_ranges[type].size()).latin1(), toHexLabelAbs(value).latin1());
+  if ( offset>=_ranges[type].size() ) qDebug("Memory::setWord: type=%s offset=%s size=%s value=%s", type.key().latin1(), toHexLabelAbs(offset).latin1(), toHexLabelAbs(_ranges[type].size()).latin1(), toHexLabelAbs(value).latin1());
   CRASH_ASSERT( offset<_ranges[type].size() );
   _ranges[type][offset] = value;
 }
@@ -459,7 +459,7 @@ void Pic::Memory::toHexBuffer(MemoryRangeType type, HexBuffer &hb) const
 {
   if ( !device().isWritable(type) ) return;
   uint nbBytes = device().architecture().data().nbBytesWord;
-  bool packed = device().architecture().data().packed;
+  bool packed = device().architecture().data().properties & PackedHex;
   uint offset = device().range(type).hexFileOffset;
   if ( offset==0 ) offset = device().range(type).start.toUInt();
   BitValue mask = device().mask(type);
@@ -505,7 +505,7 @@ void Pic::Memory::fromHexBuffer(MemoryRangeType type, const HexBuffer &hb, Warni
 {
   if ( !device().isWritable(type) ) return;
   uint nbBytes = device().architecture().data().nbBytesWord;
-  bool packed = device().architecture().data().packed;
+  bool packed = device().architecture().data().properties & PackedHex;
   uint offset = device().range(type).hexFileOffset;
   if ( offset==0 ) offset = device().range(type).start.toUInt();
   BitValue mask = device().mask(type);
