@@ -15,13 +15,6 @@
 #include "pic_debug.h"
 
 //-----------------------------------------------------------------------------
-bool Programmer::PicGroup::canReadVoltages() const
-{
-  for (uint i=0; i<Pic::Nb_VoltageTypes; i++)
-    if ( canReadVoltage(Pic::VoltageType(i)) ) return true;
-  return false;
-}
-
 Debugger::DeviceSpecific *Programmer::PicGroup::createDebuggerDeviceSpecific(::Debugger::Base &base) const
 {
   const Pic::Data *data = static_cast<const Pic::Data *>(base.device());
@@ -55,15 +48,6 @@ Programmer::PicBase::~PicBase()
   delete _deviceMemory;
 }
 
-void Programmer::PicBase::clear()
-{
-  ::Programmer::Base::clear();
-  for (uint i=0; i<Pic::Nb_VoltageTypes; i++) {
-    _voltages[i].error = false;
-    _voltages[i].value = UNKNOWN_VOLTAGE;
-  }
-}
-
 uint Programmer::PicBase::nbSteps(Task task, const Device::MemoryRange *range) const
 {
   const Pic::MemoryRange *prange = static_cast<const Pic::MemoryRange *>(range);
@@ -98,37 +82,6 @@ uint Programmer::PicBase::nbSteps(Task task, const Device::MemoryRange *range) c
   return 0;
 }
 
-bool Programmer::PicBase::readVoltages()
-{
-  if ( !hardware()->readVoltages(_voltages) ) return false;
-  bool ok = true;
-  for (uint i=0; i<Pic::Nb_VoltageTypes; i++) {
-    if ( !group().canReadVoltage(Pic::VoltageType(i)) ) continue;
-    if ( _voltages[i].error==true ) {
-      ok = false;
-      log(Log::LineType::Error, i18n("  %1 = %2 V: error in voltage level.").arg(i18n(Pic::VOLTAGE_TYPE_LABELS[i])).arg(_voltages[i].value));
-    } else if ( _voltages[i].value!=UNKNOWN_VOLTAGE )
-      log(Log::DebugLevel::Normal, QString("  %1 = %2 V").arg(i18n(Pic::VOLTAGE_TYPE_LABELS[i])).arg(_voltages[i].value));
-  }
-  return ok;
-}
-
-bool Programmer::PicBase::internalSetupHardware()
-{
-  if ( !Base::internalSetupHardware() ) return false;
-  if ( group().properties() & ::Programmer::CanReleaseReset ) {
-    log(Log::DebugLevel::Normal, "  Hold reset");
-    if ( !hardware()->setTargetReset(Pic::ResetHeld) ) return false;
-  }
-  Pic::TargetMode mode;
-  if ( !getTargetMode(mode) ) return false;
-  if ( mode!=Pic::TargetInProgramming ) {
-    log(Log::LineType::Error, i18n("Device not in programming"));
-    return false;
-  }
-  return true;
-}
-
 bool Programmer::PicBase::initProgramming(Task)
 {
 /*
@@ -183,7 +136,7 @@ bool Programmer::PicBase::initProgramming(Task)
     if ( !readCalibration() ) return false;
   }
 
-  return initProgramming();
+  return Base::initProgramming();
 }
 
 bool Programmer::PicBase::preserveCode()
@@ -200,29 +153,6 @@ bool Programmer::PicBase::preserveEeprom()
   return readRange(Pic::MemoryRangeType::Eeprom, _deviceMemory, 0);
 }
 
-bool Programmer::PicBase::internalRun()
-{
-  _state = ::Programmer::Running;
-  return hardware()->setTargetReset(Pic::ResetReleased);
-}
-
-bool Programmer::PicBase::internalStop()
-{
-  _state = ::Programmer::Stopped;
-  return hardware()->setTargetReset(Pic::ResetHeld);
-}
-
-bool Programmer::PicBase::getTargetMode(Pic::TargetMode &mode)
-{
-  return hardware()->getTargetMode(mode);
-}
-
-bool Programmer::PicBase::initProgramming()
-{
-  _state = ::Programmer::Stopped;
-  return hardware()->setTargetReset(Pic::ResetHeld);
-}
-
 //-----------------------------------------------------------------------------
 BitValue Programmer::PicBase::readDeviceId()
 {
@@ -261,13 +191,14 @@ bool Programmer::PicBase::verifyDeviceId()
     return false;
   }
   QMap<QString, Device::IdData> ids;
-  QValueVector<QString> names = group().supportedDevices();
-  for (uint k=0; k<uint(names.count()); k++) {
-    const Pic::Data *data = static_cast<const Pic::Data *>(group().deviceData(names[k]).data);
+  QStringList names = group().supportedDevices();
+  QStringList::const_iterator it;
+  for (it=names.begin(); it!=names.end(); ++it) {
+    const Pic::Data *data = static_cast<const Pic::Data *>(group().deviceData(*it).data);
     if ( data->architecture()!=device()->architecture() ) continue;
     Device::IdData idata;
-    if ( data->matchId(rawId, idata) ) ids[names[k]] = idata;
-    }
+    if ( data->matchId(rawId, idata) ) ids[*it] = idata;
+  }
   QString message;
   if ( ids.count()!=0 ) {
     log(Log::LineType::Information, i18n("Read id: %1").arg(device()->idNames(ids).join("; ")));
@@ -390,7 +321,7 @@ bool Programmer::PicBase::restoreBandGapBits()
 
 bool Programmer::PicBase::eraseAll()
 {
-  if ( !specific()->canEraseAll() ) {
+  if ( !(specific()->capabilities() & CanEraseAll) ) {
     log(Log::LineType::SoftError, i18n("The selected programmer does not support erasing the whole device."));
     return false;
   }
@@ -438,7 +369,7 @@ bool Programmer::PicBase::eraseRange(Pic::MemoryRangeType type)
 
 bool Programmer::PicBase::internalEraseRange(Pic::MemoryRangeType type)
 {
-  if ( !specific()->canEraseRange(type) && !specific()->canEraseAll() ) {
+  if ( !specific()->canEraseRange(type) && !(specific()->capabilities() & CanEraseAll) ) {
     log(Log::LineType::SoftError, i18n("The selected programmer does not support erasing neither the specified range nor the whole device."));
     return false;
   }
@@ -646,15 +577,15 @@ bool Programmer::PicBase::internalProgram(const Device::Memory &memory, const De
       return false;
     }
     if ( specific()->canEraseRange(type) ) {
-      if ( !specific()->emulatedErase() && !eraseRange(type) ) return false;
+      if ( !(specific()->capabilities() & EmulatedErase) && !eraseRange(type) ) return false;
     } else log(Log::LineType::Warning, i18n("The range cannot be erased first by the selected programmer so programming may fail..."));
     ok = programRange(type, pmemory.arrayForWriting(type));
     VerifyData vdata(NormalVerify, pmemory);
     if (ok) ok = readRange(type, 0, &vdata);
   } else {
     if ( !eprom ) {
-      if ( specific()->canEraseAll() ) {
-        if ( !specific()->emulatedErase() ) {
+      if ( specific()->capabilities() & CanEraseAll ) {
+        if ( !(specific()->capabilities() & EmulatedErase) ) {
           log(Log::LineType::Information, i18n("  Erasing device"));
           ok = ( !readConfigEntry(Config::PreserveEeprom).toBool() || preserveEeprom() );
           if (ok) ok = eraseAll();
@@ -733,7 +664,7 @@ bool Programmer::PicBase::doProgramCalibration(const Device::Array &data)
   if ( !checkProgramCalibration(data) ) return false;
   if ( !doConnectDevice() ) return false;
   log(Log::LineType::Information, i18n("Programming calibration..."));
-  emit actionMessage(i18n("Programming calibration..."));
+  _progressMonitor.setActionMessage(i18n("Programming calibration..."));
   if ( !internalProgramCalibration(data) ) return false;
   log(Log::LineType::Information, i18n("Programming calibration successful"));
   return true;
