@@ -74,15 +74,19 @@ bool getMemoryRange(MemoryRangeType type, QDomElement element)
   QString wwa = range.attribute("word_write_align");
   QString wea = range.attribute("word_erase_align");
   if ( type==MemoryRangeType::Code ) {
-    if ( data()->_architecture==Architecture::P18F || data()->_architecture==Architecture::P18J ) {
+    data()->_nbWordsCodeWrite = 16; // ### default ?
+    data()->_nbWordsCodeRowErase = 16; // ### default ?
+    if ( !wwa.isEmpty() ) {
       data()->_nbWordsCodeWrite = wwa.toUInt(&ok);
-      if ( !ok || data()->_nbWordsCodeWrite==0 || (data()->_nbWordsCodeWrite%4)!=0 ) qFatal("Missing or malformed word write align");
+      if ( !ok || data()->_nbWordsCodeWrite==0 || (data()->_nbWordsCodeWrite%4)!=0 ) qFatal("Malformed word write align");
+    } else if ( data()->_architecture==Architecture::P18F || data()->_architecture==Architecture::P18J ) {
+      qFatal("Missing word write align");
+    }
+    if ( !wea.isEmpty() ) {
       data()->_nbWordsCodeRowErase = wea.toUInt(&ok);
-      if ( !ok || (data()->_nbWordsCodeRowErase%4)!=0 ) qFatal("Missing or malformed word erase align");
-    } else {
-      if ( !wwa.isEmpty() || !wea.isEmpty() ) qFatal("word align should not be defined for this device family/subfamily");
-      data()->_nbWordsCodeWrite = 0; // #### TODO
-      data()->_nbWordsCodeRowErase = 0; // #### TODO
+      if ( !ok || (data()->_nbWordsCodeRowErase%4)!=0 ) qFatal("Malformed word erase align");
+    } else if ( data()->_architecture==Architecture::P18F || data()->_architecture==Architecture::P18J ) {
+      qFatal("Missing word erase align");
     }
   } else if ( !wwa.isEmpty() || !wea.isEmpty() ) qFatal("word align should not be defined for this memory range");
   return true;
@@ -371,11 +375,14 @@ virtual void processDevice(QDomElement device)
   } else if ( !pc.isEmpty() ) qFatal("No PC should be provided for this device architecture");
 
   QString sw = device.attribute("self_write");
-  data()->_selfWrite = (data()->_memoryTechnology!=Device::MemoryTechnology::Flash ? SelfWrite::No : data()->_architecture.data().selfWrite);
-  if ( data()->_selfWrite==SelfWrite::Nb_Types ) {
-    data()->_selfWrite = SelfWrite::fromKey(sw);
-    if ( data()->_selfWrite==SelfWrite::Nb_Types ) qFatal("Malformed or missing self-write field");
-  } else if ( !sw.isEmpty() ) qFatal("Self-write is set for the whole family or non-flash device");
+  if ( (data()->_architecture.data().properties & MaybeSelfWrite) && data()->_memoryTechnology==Device::MemoryTechnology::Flash ) {
+    if ( sw=="yes" ) data()->_selfWrite = true;
+    else if ( sw=="no" ) data()->_selfWrite = false;
+    else qFatal("Malformed or missing self-write field");
+  } else {
+    if ( !sw.isEmpty() ) qFatal("Self-write is set for the whole family or non-flash device");
+    data()->_selfWrite = (data()->_memoryTechnology!=Device::MemoryTechnology::Flash ? false : bool(data()->_architecture.data().properties & SelfWrite));
+  }
 
   // device ids
   FOR_EACH(Device::Special, special) {
