@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2006 Nicolas Hadacek <hadacek@kde.org>             *
+ *   Copyright (C) 2005-2007 Nicolas Hadacek <hadacek@kde.org>             *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -17,7 +17,6 @@
 #include <kiconloader.h>
 #include <kpushbutton.h>
 #include <klistview.h>
-#include <klocale.h>
 
 #include "devices/list/device_list.h"
 #include "devices/base/device_group.h"
@@ -25,31 +24,59 @@
 #include "progs/list/prog_list.h"
 #include "tools/list/device_info.h"
 #include "tools/list/tool_list.h"
+#include "common/gui/container.h"
 
-namespace DeviceChooser
-{
 //-----------------------------------------------------------------------------
 void DeviceChooser::Config::writeProgrammerGroup(const Programmer::Group *group)
 {
-  writeEntry("programmer", group ? group->name() : QString::null);
+  Config config;
+  config.writeEntry("programmer", group ? group->name() : QString::null);
 }
 const Programmer::Group *DeviceChooser::Config::programmerGroup()
 {
-  QString name = readEntry("programmer", QString::null);
+  Config config;
+  QString name = config.readEntry("programmer", QString::null);
   return Programmer::lister().group(name);
 }
 
 void DeviceChooser::Config::writeToolGroup(const Tool::Group *group)
 {
-  writeEntry("tool", group ? group->name() : QString::null);
+  Config config;
+  config.writeEntry("tool", group ? group->name() : QString::null);
 }
 const Tool::Group *DeviceChooser::Config::toolGroup()
 {
-  QString name = readEntry("tool", QString::null);
+  Config config;
+  QString name = config.readEntry("tool", QString::null);
   return Tool::lister().group(name);
 }
 
+void DeviceChooser::Config::writePackage(const QString &name)
+{
+  Config config;
+  config.writeEntry("package", name);
+}
+QString DeviceChooser::Config::package()
+{
+  Config config;
+  return config.readEntry("package", "<all>");
+}
+
+void DeviceChooser::Config::writeNbPins(uint nb)
+{
+  Config config;
+  config.writeEntry("nb_pins", nb);
+}
+uint DeviceChooser::Config::nbPins()
+{
+  Config config;
+  return config.readUIntEntry("nb_pins", 0);
+}
+
 //-----------------------------------------------------------------------------
+namespace DeviceChooser
+{
+
 class ListItem : public KListViewItem
 {
 public:
@@ -83,9 +110,9 @@ const DeviceChooser::ListType::Data DeviceChooser::ListType::DATA[Nb_Types] = {
   { "flat",        I18N_NOOP("Flat List")   }
 };
 
-DeviceChooser::Dialog::Dialog(const QString &device, Type type, QWidget *parent)
+DeviceChooser::Dialog::Dialog(const QString &device, QWidget *parent)
   : ::Dialog(parent, "device_chooser_dialog", true, i18n("Select a device"),
-             Ok|Close, Close, false, QSize(400, 300)), _withAuto(type==ChooseWithAuto)
+             Ok|Close, Close, false, QSize(400, 300))
 {
   setButtonOK(KGuiItem(i18n( "&Select"), "button_ok"));
   QVBoxLayout *top = new QVBoxLayout(mainWidget(), 0, 10);
@@ -102,61 +129,79 @@ DeviceChooser::Dialog::Dialog(const QString &device, Type type, QWidget *parent)
   vbox->addStretch(1);
 
   // filters
-  QFrame *frame = new QFrame(mainWidget());
-  frame->setFrameStyle(QFrame::Panel | QFrame::Raised);
-  frame->setMargin(5);
-  hbox->addWidget(frame);
-  hbox = new QHBoxLayout(frame, 10, 10);
-  QLabel *label = new QLabel(i18n("Filters:"), frame);
-  hbox->addWidget(label);
-  vbox = new QVBoxLayout(hbox);
-
-  QHBoxLayout *shbox = new QHBoxLayout(vbox);
+  Container::Grid *container = new Container::Grid(mainWidget(), Container::Raised);
+  hbox->addWidget(container);
+  QLabel *label = new QLabel(i18n("Filters:"), container);
+  container->addWidget(label, 0,0, 0,0);
+  Container::Shrinkable *fcontainer = new Container::Shrinkable(container);
+  container->addWidget(fcontainer, 0,0, 1,1);
 
   // programmer filter
-  _programmerCombo = new KeyComboBox<QString>(frame);
+  _programmerCombo = new KeyComboBox<QString>(fcontainer);
   _programmerCombo->appendItem("<all>", i18n("<Programmer>"));
   Programmer::Lister::ConstIterator pit;
   for (pit=Programmer::lister().begin(); pit!=Programmer::lister().end(); ++pit)
     _programmerCombo->appendItem(pit.key(), pit.data()->label());
-  Config config;
-  const Programmer::Group *pgroup = config.programmerGroup();
+  const Programmer::Group *pgroup = Config::programmerGroup();
   if (pgroup) _programmerCombo->setCurrentItem(pgroup->name());
   connect(_programmerCombo->widget(), SIGNAL(activated(int)), SLOT(updateList()));
-  shbox->addWidget(_programmerCombo->widget());
+  fcontainer->appendWidget(_programmerCombo->widget());
 
   // tool filter
-  _toolCombo = new KeyComboBox<QString>(frame);
+  _toolCombo = new KeyComboBox<QString>(fcontainer);
   _toolCombo->appendItem("<all>", i18n("<Toolchain>"));
   Tool::Lister::ConstIterator tit;
   for (tit=Tool::lister().begin(); tit!=Tool::lister().end(); ++tit) {
     if ( tit.data()->isCustom() ) continue;
     _toolCombo->appendItem(tit.key(), tit.data()->label());
   }
-  const Tool::Group *tgroup = config.toolGroup();
+  const Tool::Group *tgroup = Config::toolGroup();
   if (tgroup) _toolCombo->setCurrentItem(tgroup->name());
   connect(_toolCombo->widget(), SIGNAL(activated(int)), SLOT(updateList()));
-  shbox->addWidget(_toolCombo->widget());
+  fcontainer->appendWidget(_toolCombo->widget());
 
   // memory filter
-  _memoryCombo = new EnumComboBox<Device::MemoryTechnology>(i18n("<Memory Type>"), "memory_technology", frame);
+  _memoryCombo = new EnumComboBox<Device::MemoryTechnology>(i18n("<Memory Type>"), "memory_technology", fcontainer);
   connect(_memoryCombo->combo(), SIGNAL(activated(int)), SLOT(updateList()));
-  shbox->addWidget(_memoryCombo->combo());
-
-  shbox->addStretch(1);
-  shbox = new QHBoxLayout(vbox);
+  fcontainer->appendWidget(_memoryCombo->combo());
 
   // status filter
-  _statusCombo = new EnumComboBox<Device::Status>(i18n("<Status>"), "status", frame);
+  _statusCombo = new EnumComboBox<Device::Status>(i18n("<Status>"), "status", fcontainer);
   connect(_statusCombo->combo(), SIGNAL(activated(int)), SLOT(updateList()));
-  shbox->addWidget(_statusCombo->combo());
+  fcontainer->appendWidget(_statusCombo->combo());
 
   // features filter
-  _featureCombo = new EnumComboBox<Pic::Feature>(i18n("<Feature>"), "feature", frame);
+  _featureCombo = new EnumComboBox<Pic::Feature>(i18n("<Feature>"), "feature", fcontainer);
   connect(_featureCombo->combo(), SIGNAL(activated(int)), SLOT(updateList()));
-  shbox->addWidget(_featureCombo->combo());
+  fcontainer->appendWidget(_featureCombo->combo());
 
-  shbox->addStretch(1);
+  // package filter
+  _packageCombo = new KeyComboBox<QString>(fcontainer);
+  _packageCombo->appendItem("<all>", i18n("<Package>"));
+  QStringList names;
+  for (uint i=0; Device::Package::TYPE_DATA[i].name; i++) names += Device::Package::TYPE_DATA[i].name;
+  qHeapSort(names);
+  for (uint i=0; i<names.count(); i++) _packageCombo->appendItem(names[i], i18n(Device::Package::data(names[i])->label));
+  _packageCombo->setCurrentItem(Config::package());
+  connect(_packageCombo->widget(), SIGNAL(activated(int)), SLOT(updateList()));
+  fcontainer->appendWidget(_packageCombo->widget());
+
+  // no. of pins filter
+  _pinsCombo = new KeyComboBox<uint>(fcontainer);
+  _pinsCombo->appendItem(0, i18n("<No. of Pins>"));
+  QValueList<uint> list;
+  for (uint i=0; Device::Package::TYPE_DATA[i].name; i++) {
+    const Device::Package::TypeData &tdata = Device::Package::TYPE_DATA[i];
+    for (uint k=0; k<Device::Package::MAX_NB; k++) {
+      if ( tdata.nbPins[k]==0 || list.contains(tdata.nbPins[k]) ) continue;
+      list += tdata.nbPins[k];
+    }
+  }
+  qHeapSort(list);
+  for (uint i=0; i<list.count(); i++) _pinsCombo->appendItem(list[i], QString::number(list[i]));
+  _pinsCombo->setCurrentItem(Config::nbPins());
+  connect(_pinsCombo->widget(), SIGNAL(activated(int)), SLOT(updateList()));
+  fcontainer->appendWidget(_pinsCombo->widget());
 
   // list view
   QValueList<int> widths;
@@ -185,19 +230,20 @@ DeviceChooser::Dialog::Dialog(const QString &device, Type type, QWidget *parent)
 
 DeviceChooser::Dialog::~Dialog()
 {
-  Config config;
-  config.writeProgrammerGroup(programmerGroup());
-  config.writeToolGroup(toolGroup());
+  Config::writeProgrammerGroup(programmerGroup());
+  Config::writeToolGroup(toolGroup());
   _listTypeCombo->writeConfig();
   _memoryCombo->writeConfig();
   _statusCombo->writeConfig();
   _featureCombo->writeConfig();
+  Config::writePackage(_packageCombo->currentItem());
+  Config::writeNbPins(_pinsCombo->currentItem());
 }
 
 QString DeviceChooser::Dialog::device() const
 {
   QListViewItem *item = _listView->selectedItem();
-  if ( item==0 || !static_cast<ListItem *>(item)->isDevice() ) return Device::AUTO_DATA.name;
+  if ( item==0 ) return QString::null;
   return item->text(0);
 }
 
@@ -232,6 +278,8 @@ void DeviceChooser::Dialog::resetFilters()
   _memoryCombo->reset();
   _statusCombo->reset();
   _featureCombo->reset();
+  _packageCombo->setCurrentItem("<all>");
+  _pinsCombo->setCurrentItem(0);
   updateList();
 }
 
@@ -255,7 +303,6 @@ const Tool::Group *DeviceChooser::Dialog::toolGroup() const
 
 void DeviceChooser::Dialog::updateList(const QString &device)
 {
-  QValueVector<QString> list = Device::lister().supportedDevices();
   QMap<QString, KListViewItem *> groups;
   QListViewItem *selected = 0;
   const Programmer::Group *pgroup = programmerGroup();
@@ -268,10 +315,12 @@ void DeviceChooser::Dialog::updateList(const QString &device)
     _deviceView->setText(i18n("Could not detect supported devices for toolchain \"%1\". Please check installation.").arg(tgroup->label()));
     return;
   }
-  for (int i=list.count()-1; i>=0; i--) {
-    if ( pgroup && !pgroup->isSupported(list[i]) ) continue;
-    if ( tgroup && !tgroup->isSupported(list[i]) ) continue;
-    const Device::Data *data = Device::lister().data(list[i]);
+  QStringList list = Device::lister().supportedDevices();
+  QStringList::const_iterator it = list.end();
+  for (it--; it!=list.end(); it--) {
+    if ( pgroup && !pgroup->isSupported(*it) ) continue;
+    if ( tgroup && !tgroup->isSupported(*it) ) continue;
+    const Device::Data *data = Device::lister().data(*it);
     Q_ASSERT(data);
     if ( _memoryCombo->value()!=Device::MemoryTechnology::Nb_Types && data->memoryTechnology()!=_memoryCombo->value() ) continue;
     if ( _statusCombo->value()!=Device::Status::Nb_Types && data->status()!=_statusCombo->value() ) continue;
@@ -279,23 +328,34 @@ void DeviceChooser::Dialog::updateList(const QString &device)
       if ( data->group().name()!="pic" ) continue;
       if ( !static_cast<const Pic::Data *>(data)->hasFeature(_featureCombo->value()) ) continue;
     }
+    if ( _packageCombo->currentItem()!="<all>" ) {
+      QValueVector<Device::Package>::const_iterator it;
+      for (it=data->packages().begin(); it!=data->packages().end(); ++it)
+        if ( (*it).names().contains(_packageCombo->currentItem()) ) break;
+      if ( it==data->packages().end() ) continue;
+    }
+    if ( _pinsCombo->currentItem()!=0 ) {
+      QValueVector<Device::Package>::const_iterator it;
+      for (it=data->packages().begin(); it!=data->packages().end(); ++it)
+        if ( (*it).nbPins()==_pinsCombo->currentItem() ) break;
+      if ( it==data->packages().end() ) continue;
+    }
     KListViewItem *item = 0;
     switch (_listTypeCombo->value().type()) {
       case ListType::FamilyTree: {
         QString gname = data->listViewGroup();
         if ( !groups.contains(gname) )
           groups[gname] = new ListItem(_listView, gname, false, false);
-        item = new ListItem(groups[gname], list[i]);
+        item = new ListItem(groups[gname], *it);
         break;
       }
       case ListType::Flat:
-        item = new ListItem(_listView, list[i], true, true);
+        item = new ListItem(_listView, *it, true, true);
         break;
       case ListType::Nb_Types: Q_ASSERT(false); break;
     }
-    if ( device==list[i] ) selected = item;
+    if ( device==(*it) ) selected = item;
   }
-  if (_withAuto) (void)new ListItem(_listView, i18n(Device::AUTO_DATA.label), true, false);
   if ( selected==0 ) selected = _listView->firstChild();
   if (selected) {
     _listView->setSelected(selected, true);
@@ -305,44 +365,29 @@ void DeviceChooser::Dialog::updateList(const QString &device)
 }
 
 //-----------------------------------------------------------------------------
-DeviceChooser::ComboBox::ComboBox(bool withAuto, QWidget *parent)
-  : QComboBox(parent, "device_chooser_combo"), _withAuto(withAuto)
+DeviceChooser::ComboBox::ComboBox(QWidget *parent)
+  : QComboBox(parent, "device_chooser_combo")
 {
-  if (withAuto) insertItem(i18n(Device::AUTO_DATA.label));
   Device::Lister::ConstIterator it;
   for (it=Device::lister().begin(); it!=Device::lister().end(); ++it) {
-    QValueVector<QString> devices = it.data()->supportedDevices();
+    QStringList devices = it.data()->supportedDevices();
     qHeapSort(devices);
-    for (uint k=0; k<devices.count(); k++) insertItem(devices[k]);
+    QStringList::const_iterator it;
+    for (it=devices.begin(); it!=devices.end(); ++it) insertItem(*it);
   }
 }
 
-void DeviceChooser::ComboBox::setDevice(const QString &device, const Device::Data *data)
+void DeviceChooser::ComboBox::setDevice(const QString &device)
 {
-  QString text = device;
-  if ( device.isEmpty() || device==Device::AUTO_DATA.name ) {
-    if (_withAuto) text = QString::null;
-    else text = Device::lister().supportedDevices()[0];
-  }
-  if ( text.isEmpty() ) {
-    if (data) changeItem(data->name() + " " + i18n(Device::AUTO_DATA.label), 0);
-    else changeItem(i18n(Device::AUTO_DATA.label), 0);
-    setCurrentItem(0);
-  } else setCurrentText(text);
-}
-
-QString DeviceChooser::ComboBox::device() const
-{
-  if ( _withAuto && currentItem()==0 ) return Device::AUTO_DATA.name;
-  return currentText();
+  if ( !device.isEmpty() ) setCurrentText(device);
 }
 
 //-----------------------------------------------------------------------------
-DeviceChooser::Button::Button(bool withAuto, QWidget *parent)
+DeviceChooser::Button::Button(QWidget *parent)
   : QWidget(parent, "device_chooser_button")
 {
   QHBoxLayout *hbox = new QHBoxLayout(this, 0, 10);
-  _combo = new ComboBox(withAuto, this);
+  _combo = new ComboBox(this);
   connect(_combo, SIGNAL(activated(int)), SIGNAL(changed()));
   hbox->addWidget(_combo);
   KIconLoader loader;
@@ -354,7 +399,7 @@ DeviceChooser::Button::Button(bool withAuto, QWidget *parent)
 
 void DeviceChooser::Button::chooseDevice()
 {
-  Dialog dialog(_combo->device(), (_combo->withAuto() ? ChooseWithAuto : Choose), this);
+  Dialog dialog(_combo->device(), this);
   if ( !dialog.exec() || dialog.device().isEmpty() ) return;
   _combo->setDevice(dialog.device());
   emit changed();
