@@ -178,23 +178,23 @@ void ProjectManager::View::contextMenu(QListViewItem *item, const QPoint &p, int
     RootItem *ri = static_cast<RootItem *>(item);
     if ( _project==0 ) {
       if ( ri->url().isEmpty() ) {
-        pop.insertItem("piklab_createproject", i18n("New Project..."), &Main::toplevel(), SLOT(newProject()));
-        pop.insertItem("piklab_openproject", i18n("Open Project..."), &Main::toplevel(), SLOT(openProject()));
+        pop.insertItem("piklab_createproject", i18n("New Project..."), MainWindow::self(), SLOT(newProject()));
+        pop.insertItem("piklab_openproject", i18n("Open Project..."), MainWindow::self(), SLOT(openProject()));
         pop.exec(p);
       } else {
         pop.insertTitle(i18n("Standalone File"));
         pop.insertItem("configure", i18n("Configure..."));
-        if ( pop.exec(p)==1 ) Main::toplevel().configure(ConfigCenter::Standalone);
+        if ( pop.exec(p)==1 ) MainWindow::self()->configure(ConfigCenter::Standalone);
       }
     } else {
       pop.insertTitle(i18n("Project"));
-      pop.insertItem("configure", i18n("Options..."), &Main::toplevel(), SLOT(configureProject()));
-      pop.insertItem("find", i18n("Find Files..."),   &Main::toplevel(), SLOT(runKfind()));
+      pop.insertItem("configure", i18n("Options..."), MainWindow::self(), SLOT(configureProject()));
+      pop.insertItem("find", i18n("Find Files..."),   MainWindow::self(), SLOT(runKfind()));
       pop.insertSeparator();
-      pop.insertItem("piklab_compile", i18n("Build Project"), &Main::toplevel(), SLOT(buildProject()));
-      pop.insertItem("trashcan_empty", i18n("Clean Project"), &Main::toplevel(), SLOT(cleanBuild()));
+      pop.insertItem("piklab_compile", i18n("Build Project"), MainWindow::self(), SLOT(buildProject()));
+      pop.insertItem("trashcan_empty", i18n("Clean Project"), MainWindow::self(), SLOT(cleanBuild()));
       pop.insertSeparator();
-      pop.insertItem("filenew", i18n("New Source File..."), &Main::toplevel(), SLOT(newSourceFile()));
+      pop.insertItem("filenew", i18n("New Source File..."), MainWindow::self(), SLOT(newSourceFile()));
       pop.insertItem("piklab_addfile", i18n("Add Source Files..."), this, SLOT(insertSourceFiles()));
       pop.insertItem("piklab_addfile", i18n("Add Object Files..."), this, SLOT(insertObjectFiles()));
       if ( Main::currentEditor() ) pop.insertItem("piklab_addcurrentfile", i18n("Add Current File"), this, SLOT(insertCurrentFile()));
@@ -215,11 +215,11 @@ void ProjectManager::View::contextMenu(QListViewItem *item, const QPoint &p, int
       pop.exec(p);
     } else if ( group==SourceGroup || group==HeaderGroup ) {
       pop.insertTitle(i18n("Sources"));
-      pop.insertItem("filenew", i18n("New File..."), &Main::toplevel(), SLOT(newSourceFile()));
+      pop.insertItem("filenew", i18n("New File..."), MainWindow::self(), SLOT(newSourceFile()));
       pop.insertItem("piklab_addfile", i18n("Add Source Files..."), this, SLOT(insertSourceFiles()));
       pop.exec(p);
     } else if ( group==DeviceGroup ) {
-      pop.insertItem("filenew", i18n("Select Device..."), &Main::toplevel(), SLOT(showDeviceInfo()));
+      pop.insertItem("filenew", i18n("Select Device..."), MainWindow::self(), SLOT(showDeviceInfo()));
       pop.exec(p);
     }
   }
@@ -424,7 +424,7 @@ void ProjectManager::View::clicked(int button, QListViewItem *item, const QPoint
   ::BusyCursor bc;
   switch (rtti) {
     case RootRtti:
-      Main::toplevel().configureProject();
+      MainWindow::self()->configureProject();
       break;
     case HeaderRtti: {
       if ( static_cast<HeaderItem *>(item)->group()!=DeviceGroup ) break;
@@ -464,7 +464,7 @@ void ProjectManager::View::clicked(int button, QListViewItem *item, const QPoint
           } else e = new DisassemblyEditor(url, *data, this);
           addExternalFile(fi->url(), Generated);
         } else e = Main::editorManager().createEditor(fi->url().fileType(), fi->url());
-        if ( e==0 ) break; 
+        if ( e==0 ) break;
         if ( !e->open(fi->url()) ) {
 	  delete e;
 	  break;
@@ -563,7 +563,7 @@ void ProjectManager::View::modified(const PURL::Url &url)
 void ProjectManager::View::renamed(QListViewItem *item, int, const QString &text)
 {
   Q_ASSERT ( item->rtti()==DeviceRtti );
-  Main::toplevel().setDevice(text);
+  MainWindow::self()->setDevice(text);
 }
 
 void ProjectManager::View::updateGUI()
