@@ -68,28 +68,18 @@
 #include "global_config.h"
 
 //----------------------------------------------------------------------------
-KDockWidget *MainWindow::createDock(const QString &name, const QPixmap &icon,
-                                    const QString &title, const DockPosition &position)
+MainWindow *MainWindow::_self = 0;
+
+MainWindow *MainWindow::self()
 {
-  KDockWidget *dock = createDockWidget(name, icon, 0, title);
-  dock->setDockSite(KDockWidget::DockCenter);
-  DockData ddata;
-  ddata.title = title;
-  ddata.position = position;
-  ddata.dock = dock;
-  ddata.action = new ViewMenuAction(dock);
-  connect(ddata.action, SIGNAL(activated(QWidget *)), SLOT(toggleToolView(QWidget *)));
-  _docks += ddata;
-  initDockPosition(ddata);
-  return dock;
+  if ( _self==0 ) _self = new MainWindow;
+  return _self;
 }
 
 MainWindow::MainWindow()
-  : _configGenerator(0), _pikloopsProcess(0), _kfindProcess(0), _forceProgramAfterBuild(false)
+  : _configGenerator(0), _pikloopsProcess(0), _kfindProcess(0),
+    _actionAfterBuild(Programmer::ProgramAction::Nb_Types)
 {
-  Q_ASSERT( Main::_toplevel==0 );
-  Main::_toplevel = this;
-
 // status bar
     _actionStatus = new QLabel(statusBar());
     statusBar()->addWidget(_actionStatus);
@@ -97,6 +87,10 @@ MainWindow::MainWindow()
     statusBar()->addWidget(_actionProgress);
     _debugStatus = new QLabel(statusBar());
     statusBar()->addWidget(_debugStatus, 0, true);
+    _vddStatus = new VoltageStatusWidget(i18n("Vdd"), statusBar());
+    statusBar()->addWidget(_vddStatus, 0, true);
+    _vppStatus = new VoltageStatusWidget(i18n("Vpp"), statusBar());
+    statusBar()->addWidget(_vppStatus, 0, true);
     _editorStatus = new QLabel(statusBar());
     statusBar()->addWidget(_editorStatus, 0, true);
     _programmerStatus = new ProgrammerStatusWidget(statusBar());
@@ -174,8 +168,7 @@ MainWindow::MainWindow()
     Main::_compileManager->setView(Main::_compileLog);
     connect(Main::_compileManager, SIGNAL(success()), SLOT(compileSuccess()));
     connect(Main::_compileManager, SIGNAL(failure()), SLOT(compileFailure()));
-    connect(Main::_compileManager, SIGNAL(updateFile(const Compile::FileData &)),
-            SLOT(updateFile(const Compile::FileData &)));
+    connect(Main::_compileManager, SIGNAL(updateFile(const Compile::FileData &)), SLOT(updateFile(const Compile::FileData &)));
 
 // actions
     // file actions
@@ -256,7 +249,7 @@ MainWindow::MainWindow()
     (void)new KAction(i18n("&Connect"), "connect_creating", 0,
                       Programmer::manager, SLOT(connectDevice()), actionCollection(), "prog_connect");
     (void)new KToggleAction(i18n("Device Power"), "piklab_power", 0,
-                            Programmer::manager, SLOT(toggleDevicePower()), actionCollection(), "prog_power");
+                      Programmer::manager, SLOT(toggleDevicePower()), actionCollection(), "prog_power");
     (void)new KAction(i18n("&Disconnect"), "connect_no", 0,
                       Programmer::manager, SLOT(disconnectDevice()), actionCollection(), "prog_disconnect");
     (void)new KAction(i18n("&Program"), "piklab_burnchip", Qt::SHIFT + Qt::Key_F5,
@@ -281,12 +274,10 @@ MainWindow::MainWindow()
                       this , SLOT(showProgrammerSettings()), actionCollection(), "prog_settings");
 
     // debugger actions
-    (void)new KAction(i18n("&Start"), "launch", Qt::SHIFT + Qt::Key_F9,
-                      Programmer::manager, SLOT(restart()), actionCollection(), "debug_start");
+    (void)new KAction(i18n("&Start/Step"), "piklab_debug_step", 0,
+                      Programmer::manager, SLOT(step()), actionCollection(), "debug_next");
     (void)new KAction(i18n("&Run"), "piklab_run", Qt::SHIFT + Qt::Key_F9,
                       Programmer::manager, SLOT(run()), actionCollection(), "debug_run");
-    (void)new KAction(i18n("&Step"), "piklab_debug_step", 0,
-                      Programmer::manager, SLOT(step()), actionCollection(), "debug_next");
     //(void)new KAction(i18n("Step &In"), "piklab_debug_stepin",
     //             0, this, SLOT(debugStepIn()), actionCollection(), "debug_step_in");
     //(void)new KAction(i18n("Step &Out"), "piklab_debug_stepout",
@@ -309,7 +300,7 @@ MainWindow::MainWindow()
                       this , SLOT(runPikloops()), actionCollection(), "tools_pikloops");
     (void)new KAction(i18n("&Find Files..."), "find", 0,
                       this , SLOT(runKfind()), actionCollection(), "tools_kfind");
-    (void)new KAction(i18n("&Device Information..."), "info", 0,
+    (void)new KAction(i18n("&Device Selector..."), "piklab_chip", 0,
                       this , SLOT(showDeviceInfo()), actionCollection(), "tools_device_information");
     (void)new KAction(i18n("&Config Generator..."), 0, 0,
                       this , SLOT(configGenerator()), actionCollection(), "tools_config_generator");
@@ -356,6 +347,22 @@ MainWindow::MainWindow()
 MainWindow::~MainWindow()
 {}
 
+KDockWidget *MainWindow::createDock(const QString &name, const QPixmap &icon,
+                                    const QString &title, const DockPosition &position)
+{
+  KDockWidget *dock = createDockWidget(name, icon, 0, title);
+  dock->setDockSite(KDockWidget::DockCenter);
+  DockData ddata;
+  ddata.title = title;
+  ddata.position = position;
+  ddata.dock = dock;
+  ddata.action = new ViewMenuAction(dock);
+  connect(ddata.action, SIGNAL(activated(QWidget *)), SLOT(toggleToolView(QWidget *)));
+  _docks += ddata;
+  initDockPosition(ddata);
+  return dock;
+}
+
 void MainWindow::readDockConfig()
 {
   KDockMainWindow::readDockConfig(kapp->config(), "dock_config");
@@ -370,7 +377,7 @@ void MainWindow::readDockConfig()
     if ( eit==entries.end() ) initDockPosition(*it);
   }
 
-  // readDockConfig also restore the names/tooltips: what if a new version of the application changes these names...
+  // readDockConfig also restore the names/tooltips: ### what if a new version of the application changes these names...
   for(it=_docks.begin(); it!=_docks.end(); ++it) (*it).dock->setTabPageLabel((*it).title);
   QApplication::postEvent(this, new QEvent(QEvent::CaptionChange));
 }
@@ -459,14 +466,13 @@ void MainWindow::selectTool(const Tool::Group &group)
 
 void MainWindow::setDevice(const QString &device)
 {
-  if ( device==i18n(Device::AUTO_DATA.label) ) Compile::Config::setDevice(Main::project(), Device::AUTO_DATA.name);
-  else Compile::Config::setDevice(Main::project(), device);
+  Compile::Config::setDevice(Main::project(), device);
   updateGUI();
 }
 
 void MainWindow::showDeviceInfo()
 {
-  DeviceChooser::Dialog d(Main::device(), (Main::project() ? DeviceChooser::Choose : DeviceChooser::ChooseWithAuto), this);
+  DeviceChooser::Dialog d(Main::device(), this);
   if ( d.exec() ) {
     setDevice(d.device());
     updateGUI();
@@ -530,10 +536,6 @@ void MainWindow::newSourceFile()
 
 void MainWindow::newHexFile()
 {
-  if ( Main::device()==Device::AUTO_DATA.name ) {
-    MessageBox::sorry(i18n("You need to specify a device to create a new hex file."), Log::Show);
-    return;
-  }
   QString s;
   for (uint i=0; true; i++) {
     s = i18n("Hex") + (i==0 ? QString::null : QString::number(i));
@@ -635,35 +637,44 @@ void MainWindow::updateGUI()
   bool supported = (Main::deviceData() ? Main::programmerGroup().isSupported(Main::deviceData()->name()) : false);
   f.setItalic(!supported);
   _programmerStatus->widget()->setFont(f);
-  bool isProgrammer = ( Main::programmerGroup().properties() & ::Programmer::Programmer );
+  bool isProgrammer = ( Main::programmerGroup().properties() & Programmer::Programmer );
   PURL::Url purl = Main::_projectManager->projectUrl();
   bool hasHex = ( currentType==PURL::Hex || Main::_projectManager->contains(purl.toFileType(PURL::Hex)) );
-  Main::action("prog_connect")->setEnabled(isProgrammer && idle);
-  Main::action("prog_read")->setEnabled(isProgrammer && idle);
-  Main::action("prog_program")->setEnabled(isProgrammer && hasHex && idle);
-  Main::action("prog_verify")->setEnabled(isProgrammer && hasHex && idle);
-  Main::action("prog_erase")->setEnabled(isProgrammer && idle);
-  Main::action("prog_blank_check")->setEnabled(isProgrammer && idle);
-  Programmer::State pstate = (Main::programmer() ? Main::programmer()->state() : Programmer::NotConnected);
-  static_cast<KToggleAction *>(Main::action("prog_power"))->setEnabled(isProgrammer && idle && pstate!=Programmer::NotConnected);
-  Main::action("prog_disconnect")->setEnabled(isProgrammer && idle && pstate!=Programmer::NotConnected);
+  bool hasConnected = ( isProgrammer && Main::programmerGroup().properties() & Programmer::HasConnectedState );
+  Main::action("prog_connect")->setEnabled(idle && hasConnected);
+  Main::action("prog_read")->setEnabled(idle && isProgrammer);
+  Main::action("prog_program")->setEnabled(idle && isProgrammer && hasHex);
+  Main::action("prog_verify")->setEnabled(idle && isProgrammer && hasHex);
+  Main::action("prog_erase")->setEnabled(idle && isProgrammer);
+  Main::action("prog_blank_check")->setEnabled(idle && isProgrammer);
+  Programmer::State pstate = Programmer::NotConnected;
+  if ( Main::programmer() && !Main::programmer()->hasError() ) pstate = Main::programmer()->state();
+  static_cast<KToggleAction *>(Main::action("prog_power"))->setEnabled(idle && hasConnected && pstate!=Programmer::NotConnected);
+  Main::action("prog_disconnect")->setEnabled(idle && hasConnected && pstate!=Programmer::NotConnected);
   bool isDebugger = ( Main::programmerGroup().properties() & ::Programmer::Debugger );
   bool resetAvailable = ( Main::programmerGroup().properties() & Programmer::CanReleaseReset );
-  Main::action("prog_run")->setEnabled(idle && resetAvailable && !isDebugger && pstate!=Programmer::Running);
+  Main::action("prog_run")->setEnabled(idle && !isDebugger && resetAvailable && pstate!=Programmer::Running);
   Main::action("prog_stop")->setEnabled(idle && !isDebugger && pstate==Programmer::Running);
   Main::action("prog_restart")->setEnabled(idle && !isDebugger && pstate==Programmer::Running);
   const Programmer::GroupUI *pgui = static_cast<const Programmer::GroupUI *>(Main::programmerGroup().gui());
   Main::action("prog_advanced")->setEnabled(idle && pgui->hasAdvancedDialog());
+  Device::VoltageValue vdd, vpp;
+  if ( Main::programmer() && !(Main::programmerGroup().properties() & Programmer::Custom) ) {
+    vdd = static_cast<Programmer::Base *>(Main::programmer())->voltage(Device::VoltageType::TargetVdd);
+    vpp = static_cast<Programmer::Base *>(Main::programmer())->voltage(Device::VoltageType::TargetVpp);
+  }
+  _vddStatus->setVoltage(vdd);
+  _vppStatus->setVoltage(vpp);
 
   // update debugger status
   Debugger::manager->updateDevice();
-  Main::action("debug_start")->setEnabled(idle && isDebugger && pstate!=Programmer::Running && pstate!=Programmer::Halted);
-  Main::action("debug_run")->setEnabled(idle && isDebugger && pstate!=Programmer::Running && !Debugger::manager->isStepping() );
-  Main::action("debug_halt")->setEnabled(idle && isDebugger && (pstate==Programmer::Running || Debugger::manager->isStepping()) );
+  bool isStepping = ( pstate==Programmer::Halted && Debugger::manager->isStepping());
+  Main::action("debug_run")->setEnabled(idle && isDebugger && pstate!=Programmer::Running && !isStepping);
+  Main::action("debug_next")->setEnabled(idle && isDebugger && pstate!=Programmer::Running && !isStepping);
+  Main::action("debug_halt")->setEnabled(idle && isDebugger && (pstate==Programmer::Running || isStepping));
   Main::action("debug_stop")->setEnabled(idle && isDebugger && (pstate==Programmer::Running || pstate==Programmer::Halted));
-  Main::action("debug_next")->setEnabled(idle && isDebugger && pstate!=Programmer::Running);
   Main::action("debug_reset")->setEnabled(idle && isDebugger && (pstate==Programmer::Running || pstate==Programmer::Halted));
-  Main::action("debug_show_pc")->setEnabled(idle && isDebugger && Debugger::manager->coff()!=0 && Debugger::manager->pc().isInitialized() );
+  Main::action("debug_show_pc")->setEnabled(idle && isDebugger && Debugger::manager->coff()!=0 && Debugger::manager->pc().isInitialized() && pstate==Programmer::Halted && !isStepping);
 
   Main::_projectManager->updateGUI();
 
@@ -676,7 +687,7 @@ void MainWindow::updateGUI()
   }
   setCaption(KApplication::kApplication()->makeStdCaption(caption));
 
-  emit stateChanged();
+  emit deviceChanged();
 }
 
 void MainWindow::updateToolViewsActions()
@@ -747,13 +758,16 @@ void MainWindow::compileFile()
   PURL::Url url = (e ? e->url() : Main::_projectManager->selectedUrl());
   bool generated = (e ? Main::_projectManager->isExternalFile(url) : false);
   if ( Main::project()==0 || !generated ) Main::_projectManager->removeExternalFiles();
-  url = (!generated ? url : Main::_projectManager->projectUrl());
-  if ( Main::_compileManager->compileFile(Compile::TodoItem(url, generated)) ) Main::setState(Main::Compiling);
+  Compile::Item item;
+  item.url = (!generated ? url : Main::_projectManager->projectUrl());
+  item.type = (generated ? Compile::Item::Generated : Compile::Item::NotGenerated);
+  if ( Main::_compileManager->compileFile(item) ) Main::setState(Main::Compiling);
   else compileFailure();
 }
 
-void MainWindow::buildProject()
+void MainWindow::buildProject(Programmer::ProgramAction action)
 {
+  _actionAfterBuild = action;
   if ( Main::project()==0 ) {
     compileFile();
     return;
@@ -766,22 +780,34 @@ void MainWindow::buildProject()
     Editor *e = Main::_editorManager->findEditor(*it);
     if ( e && e->isModified() ) e->save();
   }
-  bool tmp = _forceProgramAfterBuild;
+  Programmer::ProgramAction tmp = _actionAfterBuild;
   stopOperations();
-  _forceProgramAfterBuild = tmp;
+  _actionAfterBuild = tmp;
   Main::_compileLog->clear();
   Main::_projectManager->removeExternalFiles();
-  Compile::LinkType ltype = (Main::programmerGroup().name()=="icd2_debugger" ? Compile::Icd2Linking : Compile::NormalLinking);
-  if ( Main::_compileManager->buildProject(ltype) ) Main::setState(Main::Compiling);
+  if ( Main::_compileManager->buildProject() ) Main::setState(Main::Compiling);
   else compileFailure();
 }
 
 void MainWindow::compileFailure()
 {
-  _forceProgramAfterBuild = false;
+  _actionAfterBuild = Programmer::ProgramAction::Nb_Types;
   Main::setState(Main::Idle);
 }
 
+void MainWindow::doAfterBuild()
+{
+  if ( _actionAfterBuild==Programmer::ProgramAction::Nb_Types && !readConfigEntry(BaseGlobalConfig::ProgramAfterBuild).toBool() ) return;
+  bool allowRun = ( _actionAfterBuild==Programmer::ProgramAction::Nb_Types && _actionAfterBuild==Programmer::ProgramAction::ProgramOnly );
+  if ( !program(allowRun) ) return;
+  switch (_actionAfterBuild.type()) {
+    case Programmer::ProgramAction::Run: Programmer::manager->run(); break;
+    case Programmer::ProgramAction::Step: Programmer::manager->run(); break;
+    case Programmer::ProgramAction::ProgramOnly:
+    case Programmer::ProgramAction::Nb_Types: break;
+  }
+}
+
 void MainWindow::compileSuccess()
 {
   if ( !Main::_compileManager->compileOnly() ) {
@@ -797,10 +823,10 @@ void MainWindow::compileSuccess()
           Main::_projectManager->addExternalFile(url, ProjectManager::Included);
         }
       }
-      if ( _forceProgramAfterBuild || readConfigEntry(BaseGlobalConfig::ProgramAfterBuild).toBool() ) program();
+      doAfterBuild();
     }
   }
-  _forceProgramAfterBuild = false;
+  _actionAfterBuild = Programmer::ProgramAction::Nb_Types;
   Main::setState(Main::Idle);
 }
 
@@ -821,10 +847,8 @@ void MainWindow::cleanBuild()
 {
   stopOperations();
   Main::_compileLog->clear();
-  if ( Main::project() ) {
-    Compile::LinkType ltype = (Main::programmerGroup().name()=="icd2_debugger" ? Compile::Icd2Linking : Compile::NormalLinking);
-    Main::_compileManager->cleanProject(ltype);
-  } else {
+  if ( Main::project() ) Main::_compileManager->cleanProject();
+  else {
     PURL::FileType type;
     PURL::Url url = Main::_projectManager->standaloneGenerator(Main::currentEditor()->url(), type);
     Main::_compileManager->cleanFile(url);
@@ -906,8 +930,7 @@ HexEditor *MainWindow::getHexEditor()
       if ( res==MessageBox::Cancel ) return 0;
     }
     if ( res==MessageBox::Yes ) {
-      _forceProgramAfterBuild = true;
-      buildProject();
+      buildProject(Programmer::ProgramAction::ProgramOnly);
       return 0;
     }
   }
@@ -930,15 +953,15 @@ void MainWindow::blankCheck()
   Programmer::manager->blankCheck(Device::MemoryRange());
 }
 
-void MainWindow::program()
+bool MainWindow::program(bool allowRun)
 {
   HexEditor *editor = getHexEditor();
-  if ( editor==0 ) return;
+  if ( editor==0 ) return false;
   if ( Main::programmerGroup().isDebugger() && !Main::_projectManager->contains(editor->url()) ) {
     MessageBox::sorry(i18n("It is not possible to start a debugging session with an hex file not generated with the current project."), Log::Show);
-    return;
+    return false;
   }
-  Programmer::manager->program(*editor->memory(), Device::MemoryRange());
+  return Programmer::manager->program(*editor->memory(), Device::MemoryRange(), allowRun);
 }
 
 void MainWindow::verify()
@@ -976,11 +999,6 @@ void MainWindow::showProgress(bool show)
 
 void MainWindow::setTotalProgress(uint nb)
 {
-  //KIO::Job *job = new KIO::SimpleJob(KURL(), 0, 0, false);
-  //int id = Observer::self()->newJob(job, true);
-  //Observer::self()->slotTotalSize(job, total);
-  //Observer::self()->slotInfoMessage(job, "test");
-  //qDebug("set total steps: %i", total);
   _actionProgress->setTotalSteps(nb);
   _actionProgress->setProgress(0);
   QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput); // #### DANGER !!!!
