@@ -25,7 +25,7 @@ private:
   virtual const Programmer::Group *programmerGroup() const { return CLI::_progGroup; }
   virtual const Device::Data *deviceData() const { return CLI::_device; }
   virtual Log::View *compileView() { return view(); }
-  virtual void updateView(bool gotoPC);
+  virtual void internalUpdateView(bool gotoPC);
   virtual bool isProjectSource(const PURL::Url &) const { return true; }
 };
 
