@@ -21,7 +21,7 @@ Q_OBJECT
 public:
   CliManager(QObject *parent) : Manager(parent), _state(Idle) {}
   Port::Description portDescription() const;
-  virtual void createProgrammer(const Device::Data *data);
+  virtual ::Programmer::Generic *createProgrammer(const Device::Data *data);
 
 private:
   State _state;
@@ -29,7 +29,7 @@ private:
   virtual const Group &group() const { return *CLI::_progGroup; }
   virtual void setState(State state) { _state = state; }
   virtual const Device::Data *device() const { return CLI::_device; }
-  virtual bool internalInitProgramming(bool debugging);
+  virtual bool internalInitProgramming(bool debugging, ProgramAction action);
   virtual bool isTargetSelfPowered() const;
 };
 
