@@ -107,7 +107,7 @@ CLI::ExitCode CLI::Main::hardwareList()
 
 CLI::ExitCode CLI::Main::deviceList()
 {
-  QValueVector<QString> devices;
+  QStringList devices;
   if ( _progGroup==0 ) {
     log(Log::LineType::Normal, i18n("Supported devices:"));
     devices = Programmer::lister().supportedDevices();
@@ -117,7 +117,8 @@ CLI::ExitCode CLI::Main::deviceList()
   }
   qHeapSort(devices);
   QString s;
-  for (uint i=0; i<uint(devices.count()); i++) s += " " + devices[i];
+  QStringList::const_iterator it;
+  for (it=devices.begin(); it!=devices.end(); ++it) s += " " + *it;
   log(Log::LineType::Normal, s + "\n");
   return OK;
 }
@@ -263,7 +264,7 @@ CLI::ExitCode CLI::Main::extractRange(const QString &range)
 
 CLI::ExitCode CLI::Main::executeCommand(const QString &command)
 {
-  Programmer::Base *programmer = Programmer::manager->programmer();
+  Programmer::Generic *programmer = Programmer::manager->programmer();
   if ( command=="connect" ) return (Programmer::manager->connectDevice() ? OK : EXEC_ERROR);
   if ( command=="disconnect" ) {
     if ( programmer==0 || programmer->state()==Programmer::NotConnected )
@@ -289,7 +290,7 @@ CLI::ExitCode CLI::Main::executeCommand(const QString &command)
   }
   if ( command=="program" ) {
     if ( _progGroup->isSoftware() ) return errorExit(i18n("Reading device memory not supported for specified programmer."), NOT_SUPPORTED_ERROR);
-    return (Programmer::manager->program(*_memory, *_range) ? OK : EXEC_ERROR);
+    return (Programmer::manager->program(*_memory, *_range, true) ? OK : EXEC_ERROR);
   }
   if ( command=="verify" ) {
     if ( _progGroup->isSoftware() )
@@ -319,8 +320,8 @@ CLI::ExitCode CLI::Main::executeCommand(const QString &command)
   if ( command=="upload_firmware" ) {
     if ( !(_progGroup->properties() & ::Programmer::CanUploadFirmware) )
       return errorExit(i18n("Uploading firmware is not supported for the specified programmer."), NOT_SUPPORTED_ERROR);
-    if ( Programmer::manager->programmer()==0 ) Programmer::manager->createProgrammer(0); // no device specified
-    return (Programmer::manager->programmer()->uploadFirmware(_hexUrl) ? OK : EXEC_ERROR);
+    if ( programmer==0 ) programmer = Programmer::manager->createProgrammer(0); // no device specified
+    return (static_cast<Programmer::Base *>(programmer)->uploadFirmware(_hexUrl) ? OK : EXEC_ERROR);
   }
   Q_ASSERT(false);
   return EXEC_ERROR;
