@@ -21,7 +21,7 @@ protected:
   virtual bool execute();
   virtual bool skip(const Device::Data &) const { return false; }
   virtual bool init(const Device::Data &) { return true; } // returns false if skipped or failed
-  virtual bool execute(const Device::Data &data) = 0; // returns false if skipped or failed
+  virtual bool executeDevice(const Device::Data &data) = 0; // returns false if skipped or failed
   virtual void cleanup(const Device::Data &) {}
   virtual void checkArguments();
 
