@@ -23,10 +23,7 @@ int main(int argc, char **argv)
   LikeBack::init(true);
   LikeBack::setServer("piklab.sourceforge.net", "/likeback/send.php");
   LikeBack::setAllowFeatureWishes(true);
-  if ( kapp->isRestored() ) RESTORE(MainWindow)
-  else {
-    MainWindow *wid = new MainWindow;
-    wid->show();
-  }
+  if ( kapp->isRestored() && KMainWindow::canBeRestored(1) ) MainWindow::self()->restore(1);
+  else MainWindow::self()->show();
   return kapp->exec();
 }
