@@ -23,9 +23,13 @@ public:
 
 private slots:
   void updateStatus();
+  void showVersionDetails();
+  void showDevicesDetails();
 
 private:
-  QLabel *_status;
+  QLabel      *_status, *_devicesLabel;
+  QStringList  _arguments;
+  QStringList  _versionLines, _devicesLines;
 };
 
 //----------------------------------------------------------------------------
