@@ -16,72 +16,75 @@
 #include "icd2_usb.h"
 
 //-----------------------------------------------------------------------------
-const uchar Icd2::TARGET_MODE_VALUES[Pic::Nb_TargetModes] = {
+const Icd2::TARGET_MODE_VALUES::Data Icd2::TARGET_MODE_VALUES::DATA[Nb_Types] = {
   0x00, // stopped
   0x01, // running
   0x02  // in programming
 };
 
-const char * const Icd2::RESET_MODE_VALUES[Pic::Nb_ResetModes] = {
+const Icd2::RESET_MODE_VALUES::Data Icd2::RESET_MODE_VALUES::DATA[Nb_Types] = {
   "00", // reset held
   "01"  // reset released
 };
 
 //-----------------------------------------------------------------------------
-const Icd2::Hardware::VoltageTypeData Icd2::Hardware::VOLTAGE_TYPE_DATA[Pic::Nb_VoltageTypes] = {
- { "37", 0.07988 }, // icd Vpp
- { "34", 0.03908 }, // target Vdd
- { "35", 0.07988 }, // target Vpp
-};
-
-const char * const Icd2::Hardware::WRITE_MODE_VALUES[Pic::Nb_WriteModes] = {
-  "00", // write only
-  "01"  // erase then write
-};
-
-const int Icd2::TestData::RESULT_TYPE_VALUES[::Programmer::Nb_ResultTypes+1] = {
+const Icd2::RESULT_TYPE_VALUES::Data Icd2::RESULT_TYPE_VALUES::DATA[Nb_Types] = {
   0x00, // pass
   0x01, // low
   0x80, // high
-  -2,   // not used
-  -1
+  -2,   // fail (unused)
 };
 
-const char * const Icd2::TestData::VOLTAGE_LABELS[Nb_VoltageTypes] = {
-  I18N_NOOP("Target Vdd"), I18N_NOOP("Module Vpp"), I18N_NOOP("MCLR ground"),
-  I18N_NOOP("MCLR Vdd"), I18N_NOOP("MCLR Vpp")
+const Icd2::TestData::VoltageType::Data Icd2::TestData::VoltageType::DATA[Nb_Types] = {
+  { I18N_NOOP("Target Vdd")   },
+  { I18N_NOOP("Module Vpp")   },
+  { I18N_NOOP("MCLR ground") },
+  { I18N_NOOP("MCLR Vdd")     },
+  { I18N_NOOP("MCLR Vpp")     }
 };
 
 Icd2::TestData::TestData()
 {
-  for (uint k=0; k<Nb_VoltageTypes; k++) _voltages[k] = -1;
+  FOR_EACH(VoltageType, type) _voltages[type] = -1;
 }
 
 Icd2::TestData::TestData(const QString &rx)
 {
-  for (uint k=0; k<Nb_VoltageTypes; k++)
-    _voltages[k] = fromHex(rx.mid(5 + 2*k, 2), 0);
+  FOR_EACH(VoltageType, type) _voltages[type] = fromHex(rx.mid(5 + 2*type.type(), 2), 0);
 }
 
 bool Icd2::TestData::pass() const
 {
-  for (uint k=0; k<Nb_VoltageTypes; k++)
-    if ( _voltages[k]!=RESULT_TYPE_VALUES[::Programmer::Pass] ) return false;
+  FOR_EACH(VoltageType, type)
+    if ( _voltages[type]!=RESULT_TYPE_VALUES::data(::Programmer::ResultType::Pass) ) return false;
   return true;
 }
 
 QString Icd2::TestData::result(VoltageType type) const
 {
-  for (uint i=0; i<=(::Programmer::Nb_ResultTypes); i++)
-    if ( _voltages[type]==RESULT_TYPE_VALUES[i] ) return i18n(::Programmer::RESULT_TYPE_LABELS[i]);
+  if ( _voltages[type]==-1 ) return "---";
+  FOR_EACH(::Programmer::ResultType, rtype)
+    if ( _voltages[type]==RESULT_TYPE_VALUES::data(rtype) ) return type.label();
   return toHex(_voltages[type], 2);
 }
 
 QString Icd2::TestData::pretty(VoltageType type) const
 {
-  return i18n(VOLTAGE_LABELS[type]) + "=" + result(type);
+  return type.label() + "=" + result(type);
 }
 
+//-----------------------------------------------------------------------------
+const Icd2::Hardware::VOLTAGE_TYPE_DATA::Data Icd2::Hardware::VOLTAGE_TYPE_DATA::DATA[Nb_Types] = {
+ { "37", 0.07988 }, // icd Vpp
+ { "34", 0.03908 }, // target Vdd
+ { "35", 0.07988 }, // target Vpp
+};
+
+const Icd2::Hardware::WRITE_MODE_VALUES::Data Icd2::Hardware::WRITE_MODE_VALUES::DATA[Nb_Types] = {
+  "00", // write only
+  "01"  // erase then write
+};
+
 //-----------------------------------------------------------------------------
 const char *Icd2::Hardware::readCommand(Pic::MemoryRangeType type) const
 {
@@ -268,38 +271,39 @@ bool Icd2::Hardware::setTargetPowerOn(bool on)
   return command(QString("05") + (on ? "FF" : "00"), 0);
 }
 
-bool Icd2::Hardware::readVoltage(Pic::VoltageType type, double &value)
+bool Icd2::Hardware::readVoltage(Device::VoltageType type, double &value)
 {
-  if ( !command(VOLTAGE_TYPE_DATA[type].command, 2) ) return false;
-  value = VOLTAGE_TYPE_DATA[type].factor * fromHex(_rx.mid(5, 2), 0);
+  if ( !command(VOLTAGE_TYPE_DATA::data(type).command, 2) ) return false;
+  value = VOLTAGE_TYPE_DATA::data(type).factor * fromHex(_rx.mid(5, 2), 0);
   return true;
 }
 
-bool Icd2::Hardware::readVoltages(VoltagesData &voltages)
+bool Icd2::Hardware::readVoltages(Device::VoltageValues &voltages)
 {
-  for (uint i=0; i<Pic::Nb_VoltageTypes; i++) {
-    if ( !readVoltage(Pic::VoltageType(i), voltages[i].value) ) return false;
-    voltages[i].error = false;
+  FOR_EACH(Device::VoltageType, type) {
+    double value;
+    if ( !readVoltage(type, value) ) return false;
+    voltages[type.type()] = Device::VoltageValue(Device::VoltageValue::Normal, value);
   }
   return true;
 }
 
-bool Icd2::Hardware::getTargetMode(Pic::TargetMode &tmode)
+bool Icd2::Hardware::getTargetMode(Device::TargetMode &tmode)
 {
   if ( !command("2C", 2) ) return false;
-  uchar mode = fromHex(_rx.mid(5, 2), 0);
-  for (uint i=0; i<Pic::Nb_TargetModes; i++) {
-    if ( mode!=TARGET_MODE_VALUES[i] ) continue;
-    tmode = Pic::TargetMode(i);
+  uchar v = fromHex(_rx.mid(5, 2), 0);
+  FOR_EACH(Device::TargetMode, mode) {
+    if ( v!=TARGET_MODE_VALUES::data(mode) ) continue;
+    tmode = mode;
     return true;
   }
   Q_ASSERT(false);
   return false;
 }
 
-bool Icd2::Hardware::setTargetReset(Pic::ResetMode mode)
+bool Icd2::Hardware::setTargetReset(Device::ResetMode mode)
 {
-  return command(QString("33") + RESET_MODE_VALUES[mode], 0);
+  return command(QString("33") + RESET_MODE_VALUES::data(mode), 0);
 }
 
 bool Icd2::Hardware::selfTest(TestData &test)
@@ -458,7 +462,7 @@ bool Icd2::Hardware::writeMemory(Pic::MemoryRangeType type, uint wordOffset, con
 
 bool Icd2::Hardware::eraseAll()
 {
-  setTargetReset(Pic::ResetHeld);
+  setTargetReset(Device::ResetMode::Held);
   if ( hasError() ) return false;
   if ( !sendCommand("29") ) return false;
   if ( !receiveResponse("29", 0, true) ) return false; // poll
@@ -480,9 +484,9 @@ bool Icd2::Hardware::resumeRun()
   return command("30", 0);
 }
 
-bool Icd2::Hardware::setWriteMode(Pic::WriteMode mode)
+bool Icd2::Hardware::setWriteMode(Device::WriteMode mode)
 {
-  return command(QString("4B") + WRITE_MODE_VALUES[mode], 0);
+  return command(QString("4B") + WRITE_MODE_VALUES::data(mode), 0);
 }
 
 bool Icd2::Hardware::writeRegister(Address address, BitValue value, uint nbBytes)
