@@ -24,8 +24,8 @@ public:
 private:
   QComboBox *_warningLevel;
   virtual void initEntries();
-  virtual void loadConfig(const Compile::Config &config);
-  virtual void saveConfig(Compile::Config &config) const;
+  virtual void internalLoadConfig(const Compile::Config &config);
+  virtual void internalSaveConfig(Compile::Config &config) const;
 };
 
 //----------------------------------------------------------------------------
