@@ -25,8 +25,8 @@ public:
 
 protected:
   QComboBox *_gpasmWarning;
-  virtual void loadConfig(const Compile::Config &config);
-  virtual void saveConfig(Compile::Config &config) const;
+  virtual void internalLoadConfig(const Compile::Config &config);
+  virtual void internalSaveConfig(Compile::Config &config) const;
 };
 
 //----------------------------------------------------------------------------
